/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dashboard2014;

import edu.wpi.first.wpilibj.networktables2.NetworkTableNode;
import edu.wpi.first.wpilibj.networktables2.util.List;
import java.awt.image.BufferedImage;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;
import java.util.prefs.Preferences;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 *
 * @author Robotics
 */
public class Backend {

    private final SliderComplex powerSlider;
    private final SliderComplex angleSlider;
    private final VideoOutput video;
    private final TeamNumChooser teamNumber;
    private final JTable prefTable;
    private int teamIP;
    private final PingTask pingTask;
    private final MainPanel panel;
    private final boolean fake_auto_hot;

    public Backend(SliderComplex c, SliderComplex ang, VideoOutput o, TeamNumChooser teamnum,
            JTable prefs, final MainPanel panel, Set<String> args) {

        if (fake_auto_hot = args.contains("--force-hot")) {
            System.out.println("Opt: --force-hot applied");
        }

        if (args.contains("--help") || args.contains("-h")) {
            System.out.println("options: --force-hot");
        }

        powerSlider = c;
        angleSlider = ang;
        video = o;
        if (args.contains("--fake-img")) {
            System.out.println("Opt: --fake-img applied");
            video.init(true);
        } else {
            video.init(false);
        }

        teamNumber = teamnum;
        prefTable = prefs;
        this.panel = panel;
        prefwatcher = new TableUpdater((PrefTableModel) prefTable.getModel());
        prefTable.getModel().addTableModelListener(prefwatcher);

        // read and set configurations
        final Preferences p = Preferences.userRoot().node("Dashboard2014-Settings");
        int team = p.getInt("TEAM_NUM", 254);

        // add listeners
        teamNumber.setNumberChangeListener(new Runnable() {
            @Override
            public void run() {
                int ip = teamNumber.getNumber();
                pingTask.changeIP(ip);
                video.setIP(ip);
                panel.setTitleTeam(ip);
                p.putInt("TEAM_NUM", ip);
            }
        });

        {
            // vision: run three cycles on test image for JIT
            BufferedImage b = FakeFrameGrabber.getRandomStoredImage();
            Vision.process(b);
            Vision.process(b);
            Vision.process(b);
        }

        teamIP = team;
        panel.setTitleTeam(team);
        video.setIP(team);
        pingTask = new PingTask(team, this);
        teamNumber.setNumber(team);
    }

    public static TableModel getPreferencesModel() {
        return new PrefTableModel();
    }

    static String pad(String a, int len, char rep) {
        int off = len - a.length();
        if (off > 0) {
            StringBuilder builder = new StringBuilder(a);
            for (int x = 0; x < off; x++) {
                builder.append(rep);
            }
            return builder.toString();
        } else {
            return a;
        }
    }

    /*
     Does _no_ range checking.
     */
    private static String getTeamIP(int num, int finaddr) {
        return String.format("10.%d.%d.%d", num / 100, num % 100, finaddr);
    }

    public static String getRobotIP(int num) {
        if (num <= 0) {
            return "localhost";
        }
        return getTeamIP(num, 2);
    }

    public static String getCameraIP(int num) {
        // g! inurl:axis-cgi/mjpg/video.cgi works to find test streams ;-)
        if (num <= 0) {
            return "localhost";
        }
        return getTeamIP(num, 11);
    }

    public int getIP() {
        teamIP = teamNumber.getNumber();
        return teamIP;
    }

    private static class TableUpdater implements TableModelListener {

        final PrefTableModel model;

        public TableUpdater(PrefTableModel b) {
            model = b;
            node = null;
        }

        @Override
        public void tableChanged(TableModelEvent e) {
            if (e.getType() == TableModelEvent.UPDATE) {
                int row = e.getFirstRow();
                String key = model.getKey(row);
                Double value = model.getValue(row);
                NetworkTableNode n = node;
                if (n != null) {
                    n.putDouble(prefHeader + key, value);
                }
            }
        }

        private volatile NetworkTableNode node;

        public void updateNode(NetworkTableNode n) {
            node = n;
        }
    }

    private final TableUpdater prefwatcher;

    public void connected(NetworkTableNode nettable) {
        System.err.println("Connected");

        List ls = nettable.getEntryStore().keys();
        for (int i = 0; i < ls.size(); i++) {
            String key = (String) ls.get(i);
            valueChanged(nettable, key, nettable.getValue(key));
        }
        prefwatcher.updateNode(nettable);
        panel.setConnected(true);
    }

    public void disconnected() {
        System.err.println("Disconnected");
        prefwatcher.updateNode(null);
        panel.setConnected(false);
    }

    public static String getDataPath(String subfolder) {
        Date d = new Date();
        DateFormat x = new SimpleDateFormat("yy-MM-dd/HH-mm-ss");
        // todo add selector
        return "dashboard-log/" + subfolder + "/" + x.format(d) + "/";
    }

    private static final String sdHeader = "/SmartDashboard/";
    private static final String prefHeader = "/Constants/";

    public void valueChanged(final NetworkTableNode nettable, String addr, Object value) {
        if (addr.startsWith(sdHeader)) {
            String key = addr.replaceFirst(sdHeader, "");
            switch (key) {
                case "flash_screen":
                    panel.flashBackground((Boolean) value);
                    break;
                case "manual_power":
                    powerSlider.set((Double) value);
                    break;
                case "intake_angle":
                    angleSlider.set((Double) value);
                    break;
                case "process_img":
                    if ((Boolean) value) {
                        // launch a worker that later sets "is_hot_goal"
                        // to true or false, and "process_img" to false
                        (new Worker() {
                            Vision.VisionResult result;

                            // seperate thread
                            @Override
                            void calculate() {
                                //BufferedImage i = video.getLastFrame();
                                System.err.println("vision start");
                                BufferedImage i = TimerFrameGrabber.readImg(teamIP);
                                if (i == null) {
                                    System.err.println("null img");
                                    result = new Vision.VisionResult();
                                    return;
                                }

                                long t = System.currentTimeMillis();
                                result = Vision.process(i);
                                System.err.println(System.currentTimeMillis() - t);
                            }

                            // sync
                            @Override
                            void finish() {
                                nettable.putBoolean(sdHeader + "img_failed", result.failed);
                                nettable.putBoolean(sdHeader + "is_hot_goal", result.hot || fake_auto_hot);
                                nettable.putBoolean(sdHeader + "process_img", false);
                            }

                            // seperate thread
                            @Override
                            void cleanup() {
                                // save/stats img to file
                                Vision.writeData(result, getDataPath("vision"));
                            }
                        }).start();
                    }
                    break;
            }
        } else if (addr.startsWith(prefHeader)) {
            String key = addr.replaceFirst(prefHeader, "");
            PrefTableModel model = (PrefTableModel) prefTable.getModel();
            if (value instanceof Double) {
                model.updatePair(key, (Double) value);
            } else {
                System.out.format("Nondouble received %s\n", value.toString());
            }

        } else {
            System.out.format("Unidentified key: %s\n", addr);
        }
    }
}
