/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package dashboard2014;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBufferInt;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;

/**
 *
 * @author Robotics
 */
public class Vision {

    public static final double HISTOGRAM_CUTOFF = 0.020;
    public static final double MIN_CANDITATE_AREA = 0.002;

    public final static boolean filterRally2014(int r, int g, int b) {
        return (g > 200 || b > 200) && r < 40 && (b < g + 30);
    }

    public final static boolean filterFLR2014(int r, int g, int b) {
        return (g > 200 || b > 200) && r < 40 && (b < g + 30);
    }

    public final static boolean filterTroy2014(int r, int g, int b) {
        return (g > 120) && (b < g - 40) && (r < 100);
    }

    public final static boolean filter2012(int r, int g, int b) {
        final int GREEN_MIN = 100;
        final int RED_DELTA = 50;
        final int BLUE_MIN = 40;
        final int BLUE_DELTA = 10;
        return (g >= GREEN_MIN && r < g - RED_DELTA && b >= BLUE_MIN && b <= (g + BLUE_DELTA));
    }

    public final static boolean filterDemo2014(int r, int g, int b) {
        // works on 2014 difference-between-images examples
        return g > 80 && b > 80 && r < 100;
    }

    private static void filterRawDataBGR(final byte[] x, final byte[] out) {
        byte low = 0x00;
        byte high = 0x01;
        for (int i = 0, j = 0; i < x.length; i += 3, j++) {
            int b = x[i] & 0xFF;
            int g = x[i + 1] & 0xFF;
            int r = x[i + 2] & 0xFF;

            boolean f = filterFLR2014(r, g, b);
            out[j] = f ? high : low;
        }
    }

    private static byte[] loadData(BufferedImage img) {
        // type is "custom": BufferedImage.TYPE_3BYTE_BGR
        return ((DataBufferByte) (img.getData().getDataBuffer())).getData();
    }

    // x and y are presized
    private static void runHistogram(final byte[] d, int[] x, int[] y) {
        int w = x.length;
        int h = y.length;
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                int v = d[i * w + j];
                x[j] += v;
                y[i] += v;
            }
        }
    }

    private static float[] filterHist(int[] u, float iscale) {
        byte cut = (byte) (u.length * HISTOGRAM_CUTOFF);
        float[] a = new float[u.length];
        for (int i = 0; i < u.length; i++) {
            if (u[i] >= cut) {
                a[i] = u[i] / iscale;
            }
        }
        return a;
    }

    public static class Range {

        private final double min;
        private final double max;
        private final double height;

        public Range(double a, double b, double avgheight) {
            if (a > b) {
                max = a;
                min = b;
            } else {
                max = b;
                min = a;
            }
            height = avgheight;
        }

        public double getAvgHeight() {
            return height;
        }

        public double getCenter() {
            return (min + max) / 2.0;
        }

        public double getWidth() {
            return (max - min);
        }

        public double area() {
            return height * getWidth();
        }

        @Override
        public String toString() {
            return String.format("Range{%f, %f: %f}", getCenter(), getWidth(), height);
        }
    }

    private static Range[] findLumps(float[] arr, int counter_size) {
        double len = arr.length;
        double counter = (double) counter_size;

        ArrayList<Range> r = new ArrayList<>();
        double sum = 0.0;
        int start = -1;
        for (int i = 0; i < arr.length; i++) {
            if (arr[i] > 0.0) {
                if (start == -1) {
                    start = i;
                }
                sum += arr[i];
            } else if (start != -1) {
                int length = i - start;
                Range candidate = new Range(start / len, i / len, sum / (length / len) / counter);
                if (candidate.area() > MIN_CANDITATE_AREA) {
                    r.add(candidate);
                }
                sum = 0.0;
                start = -1;
            }
        }

        return r.toArray(new Range[r.size()]);
    }

    public static class VisionResult {

        public final BufferedImage filtered;
        public final BufferedImage input;
        public final boolean hot;
        public final boolean failed;
        public final float[] xhist;
        public final float[] yhist;
        public final Range[] xlump;
        public final Range[] ylump;

        public VisionResult(BufferedImage f, BufferedImage i, boolean result, boolean failed,
                float[] xhist, float[] yhist, Range[] xlump, Range[] ylump) {
            this.hot = result;
            this.failed = failed;
            this.input = i;
            this.filtered = f;
            this.xhist = xhist;
            this.yhist = yhist;
            this.xlump = xlump;
            this.ylump = ylump;
        }

        public VisionResult() {
            this(null, null, false, true, null, null, null, null);
        }
    }

    public synchronized static VisionResult process(BufferedImage img) {
        int w = img.getWidth();
        int h = img.getHeight();

        byte[] imd = loadData(img);

        byte[] data = new byte[w * h];

        filterRawDataBGR(imd, data);

        int[] xi = new int[w];
        int[] yi = new int[h];
        runHistogram(data, xi, yi);

        float[] x = filterHist(xi, h);
        float[] y = filterHist(yi, w);

        Range[] xlumps = findLumps(x, h);
        Range[] ylumps = findLumps(y, w);

        int len = xlumps.length;
        boolean result = (len == 2);
        boolean failed = (len == 0) || (len > 2);
        BufferedImage mn = createMonoImage(data, w, h);

        return new VisionResult(mn, img, result, failed, x, y, xlumps, ylumps);
    }

    private static BufferedImage createMonoImage(byte[] data, int w, int h) {
        Color on = new Color(230, 120, 255);
        Color off = new Color(0, 0, 0);
        int ion = on.getRGB();
        int ioff = off.getRGB();

        BufferedImage i = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        @SuppressWarnings("MismatchedReadAndWriteOfArray")
        int[] k = ((DataBufferInt) i.getRaster().getDataBuffer()).getData();
        for (int y = 0; y < h; y++) {
            for (int x = 0; x < w; x++) {
                int index = y * w + x;
                int val = (data[index] == 0) ? ioff : ion;
                k[index] = val;
            }
        }
        return i;
    }

    public static void writeData(VisionResult e, String path) {
        new File(path).mkdirs();
        writeImage(e.input, path + "input");
        writeImage(e.filtered, path + "filtered");

        StringBuilder r = new StringBuilder();
        r.append(String.format("Hot: %b\n", e.hot));
        r.append(String.format("Failed: %b\n", e.failed));
        r.append(String.format("XHist: %s\n", Arrays.toString(e.xhist)));
        r.append(String.format("XLump: %s\n", Arrays.toString(e.xlump)));
        r.append(String.format("YHist: %s\n", Arrays.toString(e.yhist)));
        r.append(String.format("YLump: %s\n", Arrays.toString(e.ylump)));
        String s = r.toString();
        System.out.println(s);
        for (Range rr : e.xlump) {
            double area = rr.height * (rr.max - rr.min);
            System.out.println(area);
        }
        try {
            File textOut = new File(path + "analysis.txt");
            BufferedWriter b = new BufferedWriter(new FileWriter(textOut));
            b.write(s);
            b.close();
        } catch (IOException ex) {
            ex.printStackTrace(System.err);
        }
    }

    private static void writeImage(BufferedImage i, String fullpath) {
        if (i != null) {
            try {
                File f = new File(fullpath + ".png");
                ImageIO.write(i, "png", f);
            } catch (IOException ex) {
                Logger.getLogger(Vision.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}
